@@ -3,7 +3,7 @@ package Firefox::Marionette::Element;
 use strict;
 use warnings;
 
-our $VERSION = '1.03';
+our $VERSION = '1.04';
 
 sub new {
     my ( $class, $browser, %parameters ) = @_;
@@ -19,6 +19,17 @@ sub new {
     return $element;
 }
 
+sub TO_JSON {
+    my ($self) = @_;
+    my $json = {};
+    foreach my $key ( sort { $a cmp $b } keys %{$self} ) {
+        if ( $key ne 'browser' ) {
+            $json->{$key} = $self->{$key};
+        }
+    }
+    return $json;
+}
+
 sub uuid {
     my ($self) = @_;
     return $self->{ELEMENT};
@@ -270,6 +281,46 @@ sub find_partial {
     return $self->browser()->find_partial( $value, $self );
 }
 
+sub has {
+    my ( $self, $value, $using, $from ) = @_;
+    return $self->browser()->has( $value, $using, $self );
+}
+
+sub has_id {
+    my ( $self, $value, $from ) = @_;
+    return $self->browser()->has_id( $value, $self );
+}
+
+sub has_name {
+    my ( $self, $value, $from ) = @_;
+    return $self->browser()->has_name( $value, $self );
+}
+
+sub has_tag {
+    my ( $self, $value, $from ) = @_;
+    return $self->browser()->has_tag( $value, $self );
+}
+
+sub has_class {
+    my ( $self, $value, $from ) = @_;
+    return $self->browser()->has_class( $value, $self );
+}
+
+sub has_selector {
+    my ( $self, $value, $from ) = @_;
+    return $self->browser()->has_selector( $value, $self );
+}
+
+sub has_link {
+    my ( $self, $value, $from ) = @_;
+    return $self->browser()->has_link( $value, $self );
+}
+
+sub has_partial {
+    my ( $self, $value, $from ) = @_;
+    return $self->browser()->has_partial( $value, $self );
+}
+
 1;    # Magic true value required at end of module
 __END__
 
@@ -279,7 +330,7 @@ Firefox::Marionette::Element - Represents a Firefox element retrieved using the
 
 =head1 VERSION
 
-Version 1.03
+Version 1.04
 
 =head1 SYNOPSIS
 
@@ -298,18 +349,18 @@ This module handles the implementation of a Firefox Element using the Marionette
 
 =head1 SUBROUTINES/METHODS
 
-=head2 new
+=head2 attribute 
 
-returns a new L<element|Firefox::Marionette::Element>.
-
-=head2 uuid
-
-returns the browser generated UUID connected with this L<element|Firefox::Marionette::Element>.
+accepts a scalar name a parameter.  It returns the initial value of the attribute with the supplied name. Compare with the current value returned by L<property|Firefox::Marionette::Element#property> method.
 
 =head2 browser
 
 returns the L<browser|Firefox::Marionette> connected with the L<element|Firefox::Marionette::Element>.
 
+=head2 clear
+
+clears any user supplied input from the L<element|Firefox::Marionette::Element>
+
 =head2 click
 
 sends a 'click' to the L<element|Firefox::Marionette::Element>.  The browser will wait for any page load to complete or the session's L<page_load|Firefox::Marionette::Timeouts#page_load> duration to elapse before returning, which, by default is 5 minutes.  The L<click|Firefox::Marionette::Element#click> method is also used to choose an option in a select dropdown.
@@ -324,80 +375,10 @@ sends a 'click' to the L<element|Firefox::Marionette::Element>.  The browser wil
         }
     }
 
-=head2 clear
-
-clears any user supplied input from the L<element|Firefox::Marionette::Element>
-
-=head2 text
-
-returns the text that is contained by that L<element|Firefox::Marionette::Element> (if any)
-
-=head2 tag_name
-
-returns the relevant tag name.  For example 'a' or 'input'.
-
-=head2 rect
-
-returns the current L<position and size|Firefox::Marionette::Element::Rect> of the L<element|Firefox::Marionette::Element>
-
-=head2 send_keys
-
-*** DEPRECATED - see L<type|Firefox::Marionette::Element#type>. ***
-
-=head2 type
-
-accepts a scalar string as a parameter.  It sends the string to this L<element|Firefox::Marionette::Element>, such as filling out a text box. This method returns L<the browser|Firefox::Marionette> to aid in chaining methods.
-
-=head2 switch_to_shadow_root
-
-switches to this element's L<shadow root|https://www.w3.org/TR/shadow-dom/>
-
-=head2 switch_to_frame
-
-switches to this frame within the current window.
-
-=head2 attribute 
-
-accepts a scalar name a parameter.  It returns the initial value of the attribute with the supplied name. Compare with the current value returned by L<property|Firefox::Marionette::Element#property> method.
-
-=head2 property
-
-accepts a scalar name a parameter.  It returns the current value of the property with the supplied name. Compare with the initial value returned by L<attribute|Firefox::Marionette::Element#attribute> method.
-
 =head2 css
 
 accepts a scalar CSS property name as a parameter.  It returns the value of the computed style for that property.
 
-=head2 selfie
-
-returns a L<File::Temp|File::Temp> object containing a lossless PNG image screenshot of the L<element|Firefox::Marionette::Element>.
-
-accepts the following optional parameters as a hash;
-
-=over 4
-
-=item * hash - return a SHA256 hex encoded digest of the PNG image rather than the image itself
-
-=item * full - take a screenshot of the whole document unless the first L<element|Firefox::Marionette::Element> parameter has been supplied.
-
-=item * scroll - scroll to the L<element|Firefox::Marionette::Element> supplied
-
-=item * highlights - a reference to a list containing L<elements|Firefox::Marionette::Element> to draw a highlight around
-
-=back
-
-=head2 is_enabled
-
-returns true or false if the element is enabled.
-
-=head2 is_selected
-
-returns true or false if the element is selected.
-
-=head2 is_displayed
-
-returns true or false if the element is displayed.
-
 =head2 find
 
 accepts an L<xpath expression|https://en.wikipedia.org/wiki/XPath> expression> as the first parameter and returns the first L<element|Firefox::Marionette::Element> that matches this expression.
@@ -419,6 +400,8 @@ This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit>
         $element->type('Test::More');
     }
 
+If no elements are found, a L<not found|Firefox::Marionette::Exception::NotFound> exception will be thrown.  For the same functionality that returns undef if no elements are found, see the L<has|Firefox::Marionette::Element#has> method.
+
 =head2 find_id
 
 accepts an L<id|https://developer.mozilla.org/en-US/docs/Web/HTML/Global_attributes/id> as the first parameter and returns the first L<element|Firefox::Marionette::Element> with a matching 'id' property.
@@ -440,6 +423,8 @@ This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit>
         $element->type('Test::More');
     }
 
+If no elements are found, a L<not found|Firefox::Marionette::Exception::NotFound> exception will be thrown.  For the same functionality that returns undef if no elements are found, see the L<has_id|Firefox::Marionette::Element#has_id> method.
+
 =head2 find_name
 
 This method returns the first L<element|Firefox::Marionette::Element> with a matching 'name' property.
@@ -461,6 +446,8 @@ This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit>
         $element->type('Test::More');
     }
 
+If no elements are found, a L<not found|Firefox::Marionette::Exception::NotFound> exception will be thrown.  For the same functionality that returns undef if no elements are found, see the L<has_name|Firefox::Marionette::Element#has_name> method.
+
 =head2 find_class
 
 accepts a L<class name|https://developer.mozilla.org/en-US/docs/Web/HTML/Global_attributes/class> as the first parameter and returns the first L<element|Firefox::Marionette::Element> with a matching 'class' property.
@@ -482,6 +469,8 @@ This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit>
         $element->type('Test::More');
     }
 
+If no elements are found, a L<not found|Firefox::Marionette::Exception::NotFound> exception will be thrown.  For the same functionality that returns undef if no elements are found, see the L<has_class|Firefox::Marionette::Element#has_class> method.
+
 =head2 find_selector
 
 accepts a L<CSS Selector|https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_Selectors> as the first parameter and returns the first L<element|Firefox::Marionette::Element> that matches that selector.
@@ -503,6 +492,8 @@ This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit>
         $element->type('Test::More');
     }
 
+If no elements are found, a L<not found|Firefox::Marionette::Exception::NotFound> exception will be thrown.  For the same functionality that returns undef if no elements are found, see the L<has_selector|Firefox::Marionette::Element#has_selector> method.
+
 =head2 find_tag
 
 accepts a L<tag name|https://developer.mozilla.org/en-US/docs/Web/API/Element/tagName> as the first parameter and returns the first L<element|Firefox::Marionette::Element> with this tag name.
@@ -524,6 +515,8 @@ This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit>
         # do something
     }
 
+If no elements are found, a L<not found|Firefox::Marionette::Exception::NotFound> exception will be thrown. For the same functionality that returns undef if no elements are found, see the L<has_tag|Firefox::Marionette::Element#has_tag> method.
+
 =head2 find_link
 
 accepts a text string as the first parameter and returns the first link L<element|Firefox::Marionette::Element> that has a matching link text.
@@ -545,6 +538,8 @@ This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit>
         $element->click();
     }
 
+If no elements are found, a L<not found|Firefox::Marionette::Exception::NotFound> exception will be thrown.  For the same functionality that returns undef if no elements are found, see the L<has_link|Firefox::Marionette::Element#has_link> method.
+
 =head2 find_partial
 
 accepts a text string as the first parameter and returns the first link L<element|Firefox::Marionette::Element> that has a partially matching link text.
@@ -566,6 +561,214 @@ This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit>
         $element->click();
     }
 
+If no elements are found, a L<not found|Firefox::Marionette::Exception::NotFound> exception will be thrown.  For the same functionality that returns undef if no elements are found, see the L<has_partial|Firefox::Marionette::Element#has_partial> method.
+
+=head2 has
+
+accepts an L<xpath expression|https://en.wikipedia.org/wiki/XPath> as the first parameter and returns the first L<element|Firefox::Marionette::Element> that matches this expression.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout, which, by default is 0 seconds.
+
+    use Firefox::Marionette();
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+
+    my $div = $firefox->find_class('main-content');
+    if (my $element = $div->has('//input[@id="search-input"]')) {
+        $element->type('Test::More');
+    }
+
+If no elements are found, this method will return undef.  For the same functionality that throws a L<not found|Firefox::Marionette::Exception::NotFound> exception, see the L<find|Firefox::Marionette::Element#find> method.
+
+=head2 has_id
+
+accepts an L<id|https://developer.mozilla.org/en-US/docs/Web/HTML/Global_attributes/id> as the first parameter and returns the first L<element|Firefox::Marionette::Element> with a matching 'id' property.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout, which, by default is 0 seconds.
+
+    use Firefox::Marionette();
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+
+    my $div = $firefox->find_class('main-content');
+    if (my $element = $div->has_id('search-input')) {
+        $element->type('Test::More');
+    }
+
+If no elements are found, this method will return undef.  For the same functionality that throws a L<not found|Firefox::Marionette::Exception::NotFound> exception, see the L<find_id|Firefox::Marionette::Element#find_id> method.
+
+=head2 has_name
+
+This method returns the first L<element|Firefox::Marionette::Element> with a matching 'name' property.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout, which, by default is 0 seconds.
+
+    use Firefox::Marionette();
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+
+    my $div = $firefox->find_class('main-content');
+    if (my $element = $div->has_name('q')) {
+        $element->type('Test::More');
+    }
+
+If no elements are found, this method will return undef.  For the same functionality that throws a L<not found|Firefox::Marionette::Exception::NotFound> exception, see the L<find_name|Firefox::Marionette::Element#find_name> method.
+
+=head2 has_class
+
+accepts a L<class name|https://developer.mozilla.org/en-US/docs/Web/HTML/Global_attributes/class> as the first parameter and returns the first L<element|Firefox::Marionette::Element> with a matching 'class' property.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout, which, by default is 0 seconds.
+
+    use Firefox::Marionette();
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+
+    my $div = $firefox->find_class('main-content');
+    if (my $element = $div->has_class('form-control home-search-input')) {
+        $element->type('Test::More');
+    }
+
+If no elements are found, this method will return undef.  For the same functionality that throws a L<not found|Firefox::Marionette::Exception::NotFound> exception, see the L<find_class|Firefox::Marionette::Element#find_class> method.
+
+=head2 has_selector
+
+accepts a L<CSS Selector|https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_Selectors> as the first parameter and returns the first L<element|Firefox::Marionette::Element> that matches that selector.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout, which, by default is 0 seconds.
+
+    use Firefox::Marionette();
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+
+    my $div = $firefox->find_class('main-content');
+    if (my $element = $div->has_selector('input.home-search-input')) {
+        $element->type('Test::More');
+    }
+
+If no elements are found, this method will return undef.  For the same functionality that throws a L<not found|Firefox::Marionette::Exception::NotFound> exception, see the L<find_selector|Firefox::Marionette::Element#find_selector> method.
+
+=head2 has_tag
+
+accepts a L<tag name|https://developer.mozilla.org/en-US/docs/Web/API/Element/tagName> as the first parameter and returns the first L<element|Firefox::Marionette::Element> with this tag name.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout, which, by default is 0 seconds.
+
+    use Firefox::Marionette();
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+
+    my $div = $firefox->find_class('main-content');
+    if (my $element = $div->has_tag('input');
+        # do something
+    }
+
+If no elements are found, this method will return undef.  For the same functionality that throws a L<not found|Firefox::Marionette::Exception::NotFound> exception, see the L<find_tag|Firefox::Marionette::Element#find_tag> method.
+
+=head2 has_link
+
+accepts a text string as the first parameter and returns the first link L<element|Firefox::Marionette::Element> that has a matching link text.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout, which, by default is 0 seconds.
+
+    use Firefox::Marionette();
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+
+    my $div = $firefox->find_class('container-fluid');
+    if (my $element = $div->has_link('API')->click();
+        $element->click();
+    }
+
+If no elements are found, this method will return undef.  For the same functionality that throws a L<not found|Firefox::Marionette::Exception::NotFound> exception, see the L<find_link|Firefox::Marionette::Element#find_link> method.
+
+=head2 has_partial
+
+accepts a text string as the first parameter and returns the first link L<element|Firefox::Marionette::Element> that has a partially matching link text.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout, which, by default is 0 seconds.
+
+    use Firefox::Marionette();
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+
+    my $div = $firefox->find_class('container-fluid');
+    if (my $element = $div->has_partial('AP')->click();
+        $element->click();
+    }
+
+If no elements are found, this method will return undef.  For the same functionality that throws a L<not found|Firefox::Marionette::Exception::NotFound> exception, see the L<find_partial|Firefox::Marionette::Element#find_partial> method.
+
+=head2 is_enabled
+
+returns true or false if the element is enabled.
+
+=head2 is_selected
+
+returns true or false if the element is selected.
+
+=head2 is_displayed
+
+returns true or false if the element is displayed.
+
+=head2 new
+
+returns a new L<element|Firefox::Marionette::Element>.
+
+=head2 property
+
+accepts a scalar name a parameter.  It returns the current value of the property with the supplied name. Compare with the initial value returned by L<attribute|Firefox::Marionette::Element#attribute> method.
+
+=head2 rect
+
+returns the current L<position and size|Firefox::Marionette::Element::Rect> of the L<element|Firefox::Marionette::Element>
+
+=head2 send_keys
+
+*** DEPRECATED - see L<type|Firefox::Marionette::Element#type>. ***
+
+=head2 selfie
+
+returns a L<File::Temp|File::Temp> object containing a lossless PNG image screenshot of the L<element|Firefox::Marionette::Element>.
+
+accepts the following optional parameters as a hash;
+
+=over 4
+
+=item * hash - return a SHA256 hex encoded digest of the PNG image rather than the image itself
+
+=item * full - take a screenshot of the whole document unless the first L<element|Firefox::Marionette::Element> parameter has been supplied.
+
+=item * scroll - scroll to the L<element|Firefox::Marionette::Element> supplied
+
+=item * highlights - a reference to a list containing L<elements|Firefox::Marionette::Element> to draw a highlight around
+
+=back
+
+=head2 switch_to_frame
+
+switches to this frame within the current window.
+
+=head2 switch_to_shadow_root
+
+switches to this element's L<shadow root|https://www.w3.org/TR/shadow-dom/>
+
+=head2 tag_name
+
+returns the relevant tag name.  For example 'a' or 'input'.
+
+=head2 text
+
+returns the text that is contained by that L<element|Firefox::Marionette::Element> (if any)
+
+=head2 type
+
+accepts a scalar string as a parameter.  It sends the string to this L<element|Firefox::Marionette::Element>, such as filling out a text box. This method returns L<the browser|Firefox::Marionette> to aid in chaining methods.
+
+=head2 uuid
+
+returns the browser generated UUID connected with this L<element|Firefox::Marionette::Element>.
+
 =head1 DIAGNOSTICS
 
 None.
