@@ -8,7 +8,7 @@ use Firefox::Marionette::Exception::StaleElement();
 use Firefox::Marionette::Exception::InsecureCertificate();
 use Firefox::Marionette::Exception::Response();
 
-our $VERSION = '1.03';
+our $VERSION = '1.04';
 
 sub _TYPE_INDEX            { return 0 }
 sub _MESSAGE_ID_INDEX      { return 1 }
@@ -24,7 +24,7 @@ my %_known_exceptions = (
 );
 
 sub new {
-    my ( $class, $message, $parameters ) = @_;
+    my ( $class, $message, $parameters, $options ) = @_;
     my $response;
     if ( ref $message eq 'ARRAY' ) {
         $response = bless {
@@ -74,14 +74,7 @@ sub new {
         }
     }
     if ( $response->error() ) {
-        if (
-            ( $response->error()->{error} eq 'no such element' )
-            || ( $response->error()->{message} =~
-                /^Unable[ ]to[ ]locate[ ]element/smx )
-          )
-        {
-            Firefox::Marionette::Exception::NotFound->throw( $response,
-                $parameters );
+        if ( $response->_check_old_exception_cases( $parameters, $options ) ) {
         }
         elsif ( my $class = $_known_exceptions{ $response->error()->{error} } )
         {
@@ -94,6 +87,41 @@ sub new {
     return $response;
 }
 
+sub _check_old_exception_cases {
+    my ( $self, $parameters, $options ) = @_;
+    if (   ( $self->error()->{error} eq 'no such element' )
+        || ( $self->error()->{message} =~ /^Unable[ ]to[ ]locate[ ]element/smx )
+      )
+    {
+        if ( $options->{return_undef_if_no_such_element} ) {
+            $self->{ignored_exception} = 1;
+            return 1;
+        }
+        else {
+            Firefox::Marionette::Exception::NotFound->throw( $self,
+                $parameters );
+        }
+    }
+    elsif (
+        ( $self->error()->{error} eq q[] )
+        && (
+            ( $self->error()->{message} =~ /^Stale[ ]element[ ]reference$/smx )
+            || ( $self->error()->{message} =~
+                /^The[ ]element[ ]reference[ ]is[ ]stale/smx )
+        )
+      )
+    {
+        Firefox::Marionette::Exception::StaleElement->throw( $self,
+            $parameters );
+    }
+    return;
+}
+
+sub ignored_exception {
+    my ($self) = @_;
+    return $self->{ignored_exception};
+}
+
 sub type {
     my ($self) = @_;
     return $self->{type};
@@ -123,7 +151,7 @@ Firefox::Marionette::Response - Represents a Marionette protocol response
 
 =head1 VERSION
 
-Version 1.03
+Version 1.04
 
 =head1 SYNOPSIS
 
@@ -170,6 +198,10 @@ returns the error of the response or undef.
 
 returns the result value.
 
+=head2 ignored_exception
+
+returns if the response should have generated an exception but was instructed not to.
+
 =head1 DIAGNOSTICS
 
 None.
